import React, { useState, useEffect, useRef, useCallback } from "react";
import { styled } from "@mui/material/styles";
import { SimpleAccordion } from "../../common/Accordion/SimpleAccordion";
import {
  Button,
  CircularProgress,
  TextField,
  Tooltip,
  Typography,
  Autocomplete,
  Alert
} from "@mui/material";
import {
  useAllowedGitProjects,
  useNamingNames,
  useUpdateIoc
} from "../../../api/SwaggerApi";
import { useTypingTimer } from "../../common/SearchBoxFilter/TypingTimer";
import { ConfirmationDialog } from "../../dialog/ConfirmationDialog";
import { SimpleModal } from "../../common/SimpleModal/SimpleModal";
import AccessControl from "../../auth/AccessControl";

const PREFIX = "IOCDetailAdmin";

const classes = {
  textField: `${PREFIX}-textField`
};

// TODO jss-to-styled codemod: The Fragment root was replaced by div. Change the tag if needed.
const Root = styled("div")(({ theme }) => ({
  [`& .${classes.textField}`]: {
    marginBottom: theme.spacing(1)
  }
}));

export default function IOCDetailAdmin({
  ioc,
  getIOC,
  resetTab,
  buttonDisabled
}) {
  const [gitId, setGitId] = useState(ioc.gitProjectId);
  const [
    allowedGitProjects,
    getAllowedGitProjects,
    ,
    loadingAllowedGitProjects
  ] = useAllowedGitProjects([]);
  const [nameQuery, onNameKeyUp] = useTypingTimer({ interval: 500 });
  const [names, getNames, , loadingNames] = useNamingNames();
  const [name, setName] = useState({
    uuid: ioc.namingUuid,
    description: ioc.description,
    name: ioc.namingName
  });

  // for the dialog
  const [adHocDialogOpen, setAdHocDialogOpen] = useState(false);
  const [adHocDialogTitle, setAdHocDiatlogTitle] = useState();
  const [adHocDialogDescription, setAdHocDialogDescription] = useState();
  const [error, setError] = useState();
  const callbackRef = useRef();

  function onError(message) {
    setError(message);
  }

  const requiredDataMissing = useCallback(() => !gitId || !name, [gitId, name]);

  const noModification = useCallback(
    () =>
      gitId === ioc.gitProjectId &&
      name?.uuid === ioc.namingUuid &&
      name?.description === ioc.description &&
      name?.name === ioc.namingName,
    [gitId, name, ioc]
  );

  const [uioc, actionUpdateIoc] = useUpdateIoc(ioc.id, onError);

  useEffect(() => {
    getAllowedGitProjects();
  }, [getAllowedGitProjects]);

  useEffect(() => getNames(nameQuery), [nameQuery, getNames]);

  // when user clicks Submit button a dialog should open
  const onSubmit = (event) => {
    event.preventDefault();

    openModifyModal();
  };

  // Setting up dialog properties
  const openModifyModal = () => {
    setAdHocDiatlogTitle("Modifying IOC");
    setAdHocDialogDescription(
      <Root>
        <Typography style={{ display: "inline-block" }}>
          Are you sure want to modify &nbsp;
        </Typography>
        <Typography
          style={{ fontFamily: "monospace", display: "inline-block" }}
        >
          {" "}
          {ioc.namingName}?{" "}
        </Typography>
      </Root>
    );
    callbackRef.current = modifyIoc;
    setAdHocDialogOpen(true);
  };

  // what to do when modifying IOC
  const modifyIoc = () => {
    actionUpdateIoc({
      gitProjectId: gitId,
      namingUuid: name ? name.uuid : null
    });
  };

  useEffect(() => {
    if (uioc) {
      getIOC();
      resetTab();
    }
  }, [uioc, getIOC, resetTab]);

  const iocIsDeployed = ioc.activeDeployment;

  let nameDisabledTitle = "";
  if (iocIsDeployed) {
    nameDisabledTitle =
      "IOC cannot be renamed while deployed - please undeploy first";
  }

  function nameAutocomplete(disabled) {
    const isDisabled = disabled || iocIsDeployed;
    const loading = loadingNames && !isDisabled;
    return (
      <Tooltip title={nameDisabledTitle}>
        <Autocomplete
          className={classes.textField}
          autoHighlight
          id="namingName"
          options={names}
          loading={loading}
          clearOnBlur={false}
          defaultValue={name}
          getOptionLabel={(option) => {
            return option?.name;
          }}
          renderInput={(params) => (
            <TextField
              {...params}
              label="IOC name"
              variant="outlined"
              required
              InputProps={{
                ...params.InputProps,
                endAdornment: (
                  <React.Fragment>
                    {loading ? (
                      <CircularProgress
                        color="inherit"
                        size={20}
                      />
                    ) : null}
                    {params.InputProps.endAdornment}
                  </React.Fragment>
                )
              }}
            />
          )}
          onChange={(event, value, reason) => {
            setName(value);
            setError(null);
          }}
          onInputChange={(event, value, reason) => {
            event && onNameKeyUp(event.nativeEvent);
          }}
          disabled={isDisabled}
          autoSelect
        />
      </Tooltip>
    );
  }

  function gitRepoAutocomplete(disabled) {
    const isDisabled = disabled || iocIsDeployed;
    const loading = loadingAllowedGitProjects && !isDisabled;
    return (
      <Autocomplete
        className={classes.textField}
        autoHighlight
        id="gitId"
        options={allowedGitProjects}
        loading={loading}
        clearOnBlur={false}
        defaultValue={{
          id: gitId,
          url: ioc.sourceUrl
        }}
        getOptionLabel={(option) => {
          return option.url;
        }}
        onChange={(event, value, reason) => {
          setGitId(value?.id);
          setError(null);
        }}
        renderInput={(params) => (
          <TextField
            {...params}
            label="Git repository"
            variant="outlined"
            fullWidth
            required
            InputProps={{
              ...params.InputProps,
              endAdornment: (
                <React.Fragment>
                  {loading ? (
                    <CircularProgress
                      color="inherit"
                      size={20}
                    />
                  ) : null}
                  {params.InputProps.endAdornment}
                </React.Fragment>
              )
            }}
          />
        )}
        disabled={isDisabled}
        autoSelect
      />
    );
  }

  let disabledButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledButtonTitle =
      "There is an ongoing operation, you can not modify the IOC right now";
  }

  return (
    <>
      <SimpleModal
        open={adHocDialogOpen}
        setOpen={setAdHocDialogOpen}
      >
        <ConfirmationDialog
          open={adHocDialogOpen}
          setOpen={setAdHocDialogOpen}
          title={adHocDialogTitle}
          description={adHocDialogDescription}
          callback={callbackRef.current}
        />
      </SimpleModal>

      <SimpleAccordion
        summary="Modify IOC"
        defaultExpanded
      >
        <form
          onSubmit={onSubmit}
          style={{
            width: "100%",
            display: "flex",
            flexDirection: "column",
            gap: "0.5rem"
          }}
        >
          <AccessControl
            allowedRoles={["DeploymentToolAdmin"]}
            renderNoAccess={() => nameAutocomplete(true)}
          >
            {nameAutocomplete(iocIsDeployed)}
          </AccessControl>

          <AccessControl
            allowedRoles={["DeploymentToolAdmin"]}
            allowedUsersWithRoles={[
              { user: ioc.createdBy, role: "DeploymentToolIntegrator" }
            ]}
            renderNoAccess={() => gitRepoAutocomplete(true)}
          >
            {gitRepoAutocomplete(false)}
          </AccessControl>

          {error ? <Alert severity="error">{error}</Alert> : <></>}

          <br />
          <br />
          <Tooltip title={disabledButtonTitle}>
            <span>
              <Button
                color="primary"
                variant="contained"
                disabled={
                  requiredDataMissing() ||
                  noModification() ||
                  buttonDisabled ||
                  ioc.operationInProgress
                }
                type="submit"
              >
                Modify IOC
              </Button>
            </span>
          </Tooltip>
        </form>
      </SimpleAccordion>
    </>
  );
}
